#!/usr/bin/perl -w

use strict;
use warnings;

my $sequenceFileDir  = "010_sequenceFileDir";
my $partitionFileDir = "010_partitionFileDir";

my $sgeBatchFileDir  = "010_sgeBatchFileDir";
my $outDir           = "020_outDir";

system("mkdir $sgeBatchFileDir");


opendir(DIR, $sequenceFileDir) or die "Cannot open $sequenceFileDir\n";
my @dirs = readdir(DIR);
close DIR;

my $count = 0;
foreach my $inFile (@dirs) {
  next if $inFile =~ /^\./;
  next if $inFile =~ /\.reduced/;

  #### Check the presence of outputfile
  my $jobid = $inFile;
  $jobid =~ s/\..*$//;
  my $protID = $jobid;
  $jobid =~ s/(EN)[A-Z]+0+/$1/g;

  
  ##next unless ($protID eq "ENSP00000331927");  #########################
  #####last if $count  == 5;

  print "#### $count.$inFile\t";
  
  
  my $raxmlBipartitions = "RAxML_bipartitions.".$inFile;
  if(-e "$outDir/$raxmlBipartitions"){
    print "  Finished.\n";
    next;
  }

  my $raxmlBootstrap = "RAxML_bootstrap.".$inFile;
  if(-e "$raxmlBootstrap"){
    print "  Still running.\n";
    next;
  }

  $count++;

  if($count == 1000){
    print "1000 jobs qsubed. Stopped.\n";
    exit;
  }

  #print "  Making batch file.\n";  

  #next;
  
  my $sgeTemp = &variablesSR;
  
  ### Making batch file
  open(IN, "$sequenceFileDir/$inFile");
  my @inFileCont = <IN>;
  close IN;
  my ($ref_fasHash,$ref_nameArray, $seqLength) = &phyHashReader(\@inFileCont);
  my $outgroup = $ref_nameArray->[0];
  
  $sgeTemp =~ s/JOBID/$jobid/g;
  $sgeTemp =~ s/INFILE_TXT/$inFile/g;
  $sgeTemp =~ s/SEQUENCEFILEDIR/$sequenceFileDir/g;
  $sgeTemp =~ s/PARTITIONFILEDIR/$partitionFileDir/g;
  $sgeTemp =~ s/OUTGROUP/$outgroup/g;

  open(OUT, ">$sgeBatchFileDir/$inFile");
  print OUT "$sgeTemp\n";
  close OUT;

  #### qsub files
  system("qsub $sgeBatchFileDir/$inFile");
  print "\n";
  #print "$count. qsub $sgeBatchFileDir/$inFile\n";
  
}
exit;

#############################################

sub variablesSR {

  my $sgeTempSR = '
#$ -S /bin/bash 
#$ -N JOBID 
# 
#$ -q shortP 
#$ -M xxx.xxx@oist.jp
#$ -m abe 
#$ -j yes 
#$ -cwd 
#$ -pe openmpi 2
#$ -l h_vmem=1g # Maximum memory to be expected
#$ -l virtual_free=1g # Avairable mamory before start job.
#
raxmlHPC-PTHREADS-SSE3 -f a -x 12345 -p 12345 -# 100 -m GTRGAMMA -s SEQUENCEFILEDIR/INFILE_TXT -q PARTITIONFILEDIR/INFILE_TXT -o OUTGROUP -n INFILE_TXT -T 2'; 

  return($sgeTempSR);
}


sub phyHashReader {
  my $name = "";
  my $seq = "";
  my @nameArray = ( );
  my %phyHash = ( );
  my ($inFileContSR) = @_;

  foreach my $inFileLine (@$inFileContSR) {
    chomp($inFileLine);
    if($inFileLine !~ /^\d/){
      ($name,$seq) = split(/ +/,$inFileLine);
      push(@nameArray,$name);
      $phyHash{$name} = $seq;
    }
  }
  return (\%phyHash,\@nameArray);
}
