#!/usr/bin/perl -w

use strict;
use warnings;

my $sequenceDirName = "010_sequenceFileDir";
my $partFileDirName = "010_partitionFileDir";
my $sgeBatchFileDir = "010_sgeBatchFileDir";

my $seqFileDeleted  = "030_seqFileDeleted";
my $failedFileDir    = "030_failedAnalysis";


system("mkdir $seqFileDeleted") unless (-d $seqFileDeleted);
system("mkdir $failedFileDir") unless (-d $failedFileDir);


opendir(DIR, ".") or die "Cannot open current directry.\n";
my @dirs = readdir(DIR);
close DIR;

my $count = 0;
foreach my $infoFileName (@dirs) {
  next unless $infoFileName =~ /^RAxML_info\./;
  $count++;
  
  print "$count.$infoFileName\n";
  
  my $fileName_Txt = $infoFileName;
  $fileName_Txt =~ s/RAxML_info\.//;
  my $fileNameID = $fileName_Txt;
  $fileNameID =~ s/\.txt//;
  my $id = $fileNameID;
  $id =~ s/0//g;
  
  #print "fileName_Txt $fileName_Txt\n";
  #print "fileNameID $fileNameID\n";
  #print "id $id\n";
  
  
  ### Outfile Check
  open(INFO, "$infoFileName");
  my @raxInfoCont = <INFO>;
  close INFO;

  ### Making seq file without error seq
  if(my @errorSeq = grep(/ERROR: Sequence /,@raxInfoCont)) {
    print "  Found ERROR Sequence\n";
    &erroSeqDeleter($fileName_Txt, \@errorSeq);

    ### Making sge file and qsub it
    &sgeMaker($fileName_Txt);
  
    #### qsub files
    print "  mv *$fileNameID* $failedFileDir\n";
    system("mv *$fileNameID* $failedFileDir");
    system("mv *$id* $failedFileDir");
    print "  qsub $sgeBatchFileDir/$fileName_Txt\n";
    system("qsub $sgeBatchFileDir/$fileName_Txt");
  
  } else {
    print "  No need to reanalize.\n";
  }

}
exit;

#############################################

sub erroSeqDeleter {
  my ($inFile, $ref_errorSeq) = @_;
  my @errorSeqs = @$ref_errorSeq;

  ## Collecting error seqs
  my @erroSeqIDs = ();
  foreach my $erSeq (@errorSeqs) {
    $erSeq =~ s/^.*?Sequence //;
    $erSeq =~ s/ .*$//;
    #print "  RAxML error seq: $erSeq\n";
    push(@erroSeqIDs, $erSeq);
  }
  
  ## Making error seq deleted infile
  open(RAIN, "$sequenceDirName/$inFile") or die "Cannot open $sequenceDirName/$inFile\n";
  my @raxInfileCont = <RAIN>;
  close RAIN;
  
  open(DELIN, ">$seqFileDeleted/$inFile") or die;
  DELINLOOP:foreach my $raxinline (@raxInfileCont) {
    chomp($raxinline);
    if($raxinline =~ /^\d/){
      my($spNum, $seqLength) = split(/ /,$raxinline);
      $spNum = $spNum - scalar(@errorSeqs);
      print DELIN "$spNum $seqLength\n";
      next DELINLOOP;
    }
    foreach my $erSeq (@erroSeqIDs) {
      chomp($erSeq);
      #print "  $erSeq\n";
      next DELINLOOP if $raxinline =~ /$erSeq/;
    }
    print DELIN "$raxinline\n";
  }
  close DELIN;
    
}


sub variablesSR {

  my $sgeTempSR = '
#$ -S /bin/bash 
#$ -N JOBID 
# 
#$ -q long 
#$ -M xxxx@oist.jp
#$ -m abe 
#$ -j yes 
#$ -cwd 
#$ -pe openmpi 4
#$ -l h_vmem=1g # Maximum memory to be expected
#$ -l virtual_free=256m # Avairable mamory before start job.
#
raxmlHPC-PTHREADS-SSE3 -f a -x 12345 -p 12345 -# 100 -m GTRGAMMA -s SEQUENCEFILEDIR/INFILE_TXT -q PARTITIONFILEDIR/INFILE_TXT -o OUTGROUP -n INFILE_TXT -T 4  ';

  return($sgeTempSR);
}


sub phyHashReader {
  my $name = "";
  my $seq = "";
  my @nameArray = ( );
  my %phyHash = ( );
  my ($inFileContSR) = @_;

  foreach my $inFileLine (@$inFileContSR) {
    chomp($inFileLine);
    if($inFileLine !~ /^\d/){
      ($name,$seq) = split(/ +/,$inFileLine);
      push(@nameArray,$name);
      $phyHash{$name} = $seq;
    }
  }
  return (\%phyHash,\@nameArray);
}


sub sgeMaker{

  my ($inFile) = @_;
  
  my $sgeTemp = &variablesSR;
  
  ### Printout
  open(IN, "$sequenceDirName/$inFile");
  my @inFileCont = <IN>;
  close IN;
  my ($ref_fasHash,$ref_nameArray, $seqLength) = &phyHashReader(\@inFileCont);
  my $outgroup = $ref_nameArray->[0];
  

  ### Making batch file
  my $jobid = $inFile;
  $jobid =~ s/\..*$//;
  $jobid =~ s/0//g;
  $sgeTemp =~ s/JOBID/$jobid/g;
  $sgeTemp =~ s/INFILE_TXT/$inFile/g;
  $sgeTemp =~ s/SEQUENCEFILEDIR/$seqFileDeleted/g;
  $sgeTemp =~ s/PARTITIONFILEDIR/$partFileDirName/g;
  $sgeTemp =~ s/OUTGROUP/$outgroup/g;

  open(OUT, ">$sgeBatchFileDir/$inFile");
  print OUT "$sgeTemp\n";
  close OUT;


}
